@@ -7,7 +7,8 @@ import           Control.Monad.Trans.Reader
 import           DBus.Client as C
 import qualified DBus.Internal.Message as M
 import qualified DBus.Internal.Types as T
-import qualified DBus.Introspection as I
+import qualified DBus.Introspection.Parse as I
+import qualified DBus.Introspection.Types as I
 import qualified Data.ByteString as BS
 import qualified Data.Char as Char
 import           Data.Coerce
@@ -18,11 +19,11 @@ import qualified Data.Map as Map
 import           Data.Maybe
 import           Data.Monoid
 import           Data.String
+import qualified Data.Text.IO as Text
 import           Data.Traversable
 import           Data.Word
 import           Language.Haskell.TH
 import           Prelude hiding (mapM)
-import           System.IO.Unsafe
 import           System.Posix.Types (Fd(..))
 
 type ClientBusPathR a = ReaderT (Client, T.BusName, T.ObjectPath) IO a
@@ -356,7 +357,7 @@ generateClientProperty GenerationParams
           buildSignature $ AppT (ConT ''IO) $
                          AppT (AppT (ConT ''Either)
                                       (ConT ''M.MethodError)) $ getArgType propType
-        setterSigType = buildSignature $
+        setterSigType = buildSignature $ addTypeArg (getArgType propType) $
                         AppT (ConT ''IO) $ AppT (ConT ''Maybe) (ConT ''M.MethodError)
         buildArgs rest = clientN:addArgIf takeBusArg busN
                          (addArgIf takeObjectPathArg objectPathN rest)
@@ -535,10 +536,10 @@ generateSignal GenerationParams
                         ]
 
 generateFromFilePath :: GenerationParams -> FilePath -> Q [Dec]
-generateFromFilePath generationParams filepath =
-  let obj = unsafePerformIO $
-            head . maybeToList . I.parseXML "/" <$> readFile filepath
-      interface = head $ I.objectInterfaces obj
-      signals = generateSignalsFromInterface generationParams interface
-      client = generateClient generationParams interface
-  in fmap (++) signals <*> client
+generateFromFilePath generationParams filepath = do
+    xml <- runIO $ Text.readFile filepath
+    let obj = head $ maybeToList $ I.parseXML "/" xml
+        interface = head $ I.objectInterfaces obj
+        signals = generateSignalsFromInterface generationParams interface
+        client = generateClient generationParams interface
+     in fmap (++) signals <*> client
