@@ -1,15 +1,12 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE IncoherentInstances #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell #-}
-{-# LANGUAGE TypeSynonymInstances #-}
 
 -- Copyright (C) 2009-2012 John Millikin <john@john-millikin.com>
---
--- This program is free software: you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
 -- the Free Software Foundation, either version 3 of the License, or
 -- any later version.
@@ -25,13 +22,11 @@
 module DBus.Internal.Types where
 
 import           Control.DeepSeq
-import           Control.Exception (Exception, handle, throwIO)
 import           Control.Monad (liftM, when, (>=>))
-import qualified Data.ByteString
-import           Data.ByteString (ByteString)
-import qualified Data.ByteString.Char8 as Char8
-import qualified Data.ByteString.Lazy
-import qualified Data.ByteString.Unsafe
+import           Control.Monad.Catch
+import qualified Data.ByteString as BS
+import qualified Data.ByteString.Char8 as BS8
+import qualified Data.ByteString.Lazy as BL
 import           Data.Char (ord)
 import           Data.Coerce
 import           Data.Int
@@ -43,14 +38,12 @@ import qualified Data.String
 import qualified Data.Text
 import           Data.Text (Text)
 import qualified Data.Text.Lazy
-import           Data.Typeable (Typeable)
+import           Data.Typeable (Typeable, Proxy(..))
 import qualified Data.Vector
 import           Data.Vector (Vector)
 import           Data.Word
-import qualified Foreign
 import           GHC.Generics
 import qualified Language.Haskell.TH.Lift as THL
-import           System.IO.Unsafe (unsafePerformIO)
 import           System.Posix.Types (Fd)
 import           Text.ParserCombinators.Parsec ((<|>), oneOf)
 import qualified Text.ParserCombinators.Parsec as Parsec
@@ -155,12 +148,12 @@ instance Data.String.IsString Signature where
 
 -- | Convert a list of types into a valid signature.
 --
--- Returns @Nothing@ if the given types are not a valid signature.
-signature :: [Type] -> Maybe Signature
+-- Throws if the given types are not a valid signature.
+signature :: MonadThrow m => [Type] -> m Signature
 signature = check where
     check ts = if sumLen ts > 255
-        then Nothing
-        else Just (Signature ts)
+        then throwM $ userError "invalid signature"
+        else pure (Signature ts)
     sumLen :: [Type] -> Int
     sumLen = sum . map len
 
@@ -188,29 +181,29 @@ signature_ ts = case signature ts of
 
 -- | Parse a signature string into a valid signature.
 --
--- Returns @Nothing@ if the given string is not a valid signature.
-parseSignature :: String -> Maybe Signature
+-- Throws if the given string is not a valid signature.
+parseSignature :: MonadThrow m => String -> m Signature
 parseSignature s = do
-    when (length s > 255) Nothing
-    when (any (\c -> ord c > 0x7F) s) Nothing
-    parseSignatureBytes (Char8.pack s)
+    when (length s > 255) $ throwM $ userError "string too long"
+    when (any (\c -> ord c > 0x7F) s) $ throwM $ userError "invalid signature"
+    parseSignatureBytes (BS8.pack s)
 
-parseSignatureBytes :: ByteString -> Maybe Signature
+parseSignatureBytes :: MonadThrow m => BS.ByteString -> m Signature
 parseSignatureBytes bytes =
-    case Data.ByteString.length bytes of
-        0 -> Just (Signature [])
+    case BS.length bytes of
+        0 -> pure (Signature [])
         1 -> parseSigFast bytes
         len | len <= 255 -> parseSigFull bytes
-        _ -> Nothing
+        _ -> throwM $ userError "string too long"
 
-parseSigFast :: ByteString -> Maybe Signature
+parseSigFast :: MonadThrow m => BS.ByteString -> m Signature
 parseSigFast bytes =
-    let byte = Data.ByteString.Unsafe.unsafeHead bytes in
-    parseAtom (fromIntegral byte)
-        (\t -> Just (Signature [t]))
-        (case byte of
-            0x76 -> Just (Signature [TypeVariant])
-            _ -> Nothing)
+    let byte = BS.head bytes
+     in parseAtom (fromIntegral byte)
+            (\t -> pure (Signature [t]))
+            (case byte of
+                0x76 -> pure (Signature [TypeVariant])
+                _ -> throwM $ userError "invalid signature")
 
 parseAtom :: Int -> (Type -> a) -> a -> a
 parseAtom byte yes no = case byte of
@@ -235,99 +228,90 @@ data SigParseError = SigParseError
 
 instance Exception SigParseError
 
-peekWord8AsInt :: Foreign.Ptr Word8 -> Int -> IO Int
-peekWord8AsInt ptr off = do
-    w <- Foreign.peekElemOff ptr off
-    return (fromIntegral w)
+peekWord8AsInt :: BS.ByteString -> Int -> Int
+peekWord8AsInt str i = fromIntegral $ BS.index str i
 
-parseSigFull :: ByteString -> Maybe Signature
-parseSigFull bytes = unsafePerformIO io where
-    io = handle
-        (\SigParseError -> return Nothing)
-        $ Data.ByteString.Unsafe.unsafeUseAsCStringLen bytes
-        $ \(ptr, len) -> do
-            ts <- parseSigBuf (Foreign.castPtr ptr, len)
-            return (Just (Signature ts))
+parseSigFull :: MonadThrow m => BS.ByteString -> m Signature
+parseSigFull bytes = Signature <$> mainLoop [] 0
+  where
+    len = BS.length bytes
+    mainLoop acc ii | ii >= len = pure (reverse acc)
+    mainLoop acc ii = do
+        let c = peekWord8AsInt bytes ii
+        let next t = mainLoop (t : acc) (ii + 1)
+        parseAtom c next $ case c of
+            0x76 -> next TypeVariant
+            0x28 -> do -- '('
+                (ii', t) <- structure (ii + 1)
+                mainLoop (t : acc) ii'
+            0x61 -> do -- 'a'
+                (ii', t) <- array (ii + 1)
+                mainLoop (t : acc) ii'
+            _ -> throwM SigParseError
 
-    parseSigBuf (buf, len) = mainLoop [] 0 where
-
-        mainLoop acc ii | ii >= len = return (reverse acc)
-        mainLoop acc ii = do
-            c <- peekWord8AsInt buf ii
-            let next t = mainLoop (t : acc) (ii + 1)
+    structure = loop [] where
+        loop _ ii | ii >= len = throwM SigParseError
+        loop acc ii = do
+            let c = peekWord8AsInt bytes ii
+            let next t = loop (t : acc) (ii + 1)
             parseAtom c next $ case c of
                 0x76 -> next TypeVariant
                 0x28 -> do -- '('
                     (ii', t) <- structure (ii + 1)
-                    mainLoop (t : acc) ii'
+                    loop (t : acc) ii'
                 0x61 -> do -- 'a'
                     (ii', t) <- array (ii + 1)
-                    mainLoop (t : acc) ii'
-                _ -> throwIO SigParseError
+                    loop (t : acc) ii'
+                -- ')'
+                0x29 -> case acc of
+                    [] -> throwM SigParseError
+                    _ -> pure (ii + 1, TypeStructure (reverse acc))
+                _ -> throwM SigParseError
 
-        structure :: Int -> IO (Int, Type)
-        structure = loop [] where
-            loop _ ii | ii >= len = throwIO SigParseError
-            loop acc ii = do
-                c <- peekWord8AsInt buf ii
-                let next t = loop (t : acc) (ii + 1)
-                parseAtom c next $ case c of
-                    0x76 -> next TypeVariant
-                    0x28 -> do -- '('
-                        (ii', t) <- structure (ii + 1)
-                        loop (t : acc) ii'
-                    0x61 -> do -- 'a'
-                        (ii', t) <- array (ii + 1)
-                        loop (t : acc) ii'
-                    -- ')'
-                    0x29 -> case acc of
-                        [] -> throwIO SigParseError
-                        _ -> return (ii + 1, TypeStructure (reverse acc))
-                    _ -> throwIO SigParseError
+    array ii | ii >= len = throwM SigParseError
+    array ii = do
+        let c = peekWord8AsInt bytes ii
+        let next t = pure (ii + 1, TypeArray t)
+        parseAtom c next $ case c of
+            0x76 -> next TypeVariant
+            0x7B -> dict (ii + 1) -- '{'
+            0x28 -> do -- '('
+                (ii', t) <- structure (ii + 1)
+                pure (ii', TypeArray t)
+            0x61 -> do -- 'a'
+                (ii', t) <- array (ii + 1)
+                pure (ii', TypeArray t)
+            _ -> throwM SigParseError
 
-        array :: Int -> IO (Int, Type)
-        array ii | ii >= len = throwIO SigParseError
-        array ii = do
-            c <- peekWord8AsInt buf ii
-            let next t = return (ii + 1, TypeArray t)
-            parseAtom c next $ case c of
-                0x76 -> next TypeVariant
-                0x7B -> dict (ii + 1) -- '{'
-                0x28 -> do -- '('
-                    (ii', t) <- structure (ii + 1)
-                    return (ii', TypeArray t)
-                0x61 -> do -- 'a'
-                    (ii', t) <- array (ii + 1)
-                    return (ii', TypeArray t)
-                _ -> throwIO SigParseError
+    dict ii | ii + 1 >= len = throwM SigParseError
+    dict ii = do
+        let c1 = peekWord8AsInt bytes ii
+        let c2 = peekWord8AsInt bytes (ii + 1)
 
-        dict :: Int -> IO (Int, Type)
-        dict ii | ii + 1 >= len = throwIO SigParseError
-        dict ii = do
-            c1 <- peekWord8AsInt buf ii
-            c2 <- peekWord8AsInt buf (ii + 1)
+        let next t = pure (ii + 2, t)
+        (ii', t2) <- parseAtom c2 next $ case c2 of
+            0x76 -> next TypeVariant
+            0x28 -> structure (ii + 2) -- '('
+            0x61 -> array (ii + 2) -- 'a'
+            _ -> throwM SigParseError
 
-            let next t = return (ii + 2, t)
-            (ii', t2) <- parseAtom c2 next $ case c2 of
-                0x76 -> next TypeVariant
-                0x28 -> structure (ii + 2) -- '('
-                0x61 -> array (ii + 2) -- 'a'
-                _ -> throwIO SigParseError
-
-            if ii' >= len
-                then throwIO SigParseError
-                else do
-                    c3 <- peekWord8AsInt buf ii'
-                    if c3 == 0x7D
-                        then do
-                            t1 <- parseAtom c1 return (throwIO SigParseError)
-                            return (ii' + 1, TypeDictionary t1 t2)
-                        else throwIO SigParseError
+        if ii' >= len
+            then throwM SigParseError
+            else do
+                let c3 = peekWord8AsInt bytes ii'
+                if c3 == 0x7D
+                    then do
+                        t1 <- parseAtom c1 pure (throwM SigParseError)
+                        pure (ii' + 1, TypeDictionary t1 t2)
+                    else throwM SigParseError
 
 extractFromVariant :: IsValue a => Variant -> Maybe a
 extractFromVariant (Variant (ValueVariant v)) = extractFromVariant v
 extractFromVariant v = fromVariant v
 
+typeOf :: forall a. IsValue a => a -> Type
+typeOf _ = typeOf' (Proxy :: Proxy a)
+
 class IsVariant a where
     toVariant :: a -> Variant
     fromVariant :: Variant -> Maybe a
@@ -338,7 +322,7 @@ class IsVariant a where
 -- Users may not provide new instances of 'IsValue' because this could allow
 -- containers to be created with items of heterogenous types.
 class IsVariant a => IsValue a where
-    typeOf :: a -> Type
+    typeOf' :: Proxy a -> Type
     toValue :: a -> Value
     fromValue :: Value -> Maybe a
 
@@ -359,7 +343,7 @@ newtype Variant = Variant Value
 data Value
     = ValueAtom Atom
     | ValueVariant Variant
-    | ValueBytes ByteString
+    | ValueBytes BS.ByteString
     | ValueVector Type (Vector Value)
     | ValueMap Type Type (Map Atom Value)
     | ValueStructure [Value]
@@ -427,8 +411,8 @@ showValue _ (ValueStructure xs) = showThings "(" (showValue False) ")" xs
 showThings :: String -> (a -> String) -> String -> [a] -> String
 showThings a s z xs = a ++ intercalate ", " (map s xs) ++ z
 
-vectorToBytes :: Vector Value -> ByteString
-vectorToBytes = Data.ByteString.pack
+vectorToBytes :: Vector Value -> BS.ByteString
+vectorToBytes = BS.pack
               . Data.Vector.toList
               . Data.Vector.map (\(ValueAtom (AtomWord8 x)) -> x)
 
@@ -472,7 +456,7 @@ atomType (AtomObjectPath _) = TypeObjectPath
     ; fromAtom _ = Nothing \
     }; \
     instance IsValue HsType where \
-    { typeOf _ = TypeCons \
+    { typeOf' _ = TypeCons \
     ; toValue = ValueAtom . toAtom \
     ; fromValue (ValueAtom x) = fromAtom x \
     ; fromValue _ = Nothing \
@@ -497,7 +481,7 @@ IS_ATOM(Signature,  AtomSignature,  TypeSignature)
 IS_ATOM(ObjectPath, AtomObjectPath, TypeObjectPath)
 
 instance IsValue Variant where
-    typeOf _ = TypeVariant
+    typeOf' _ = TypeVariant
     toValue = ValueVariant
     fromValue (ValueVariant x) = Just x
     fromValue _ = Nothing
@@ -511,7 +495,7 @@ instance IsAtom Data.Text.Lazy.Text where
     fromAtom = fmap Data.Text.Lazy.fromStrict . fromAtom
 
 instance IsValue Data.Text.Lazy.Text where
-    typeOf _ = TypeString
+    typeOf' _ = TypeString
     toValue = ValueAtom . toAtom
     fromValue (ValueAtom x) = fromAtom x
     fromValue _ = Nothing
@@ -525,7 +509,7 @@ instance IsAtom String where
     fromAtom = fmap Data.Text.unpack . fromAtom
 
 instance IsValue String where
-    typeOf _ = TypeString
+    typeOf' _ = TypeString
     toValue = ValueAtom . toAtom
     fromValue (ValueAtom x) = fromAtom x
     fromValue _ = Nothing
@@ -535,20 +519,19 @@ instance IsVariant String where
     fromVariant (Variant val) = fromValue val
 
 instance IsValue a => IsValue (Vector a) where
-    typeOf v = TypeArray (vectorItemType v)
-    toValue v = ValueVector (vectorItemType v) (Data.Vector.map toValue v)
+    typeOf' _ = TypeArray (typeOf' (Proxy :: Proxy a))
+    toValue v = ValueVector
+        (typeOf' (Proxy :: Proxy a))
+        (Data.Vector.map toValue v)
     fromValue (ValueVector _ v) = Data.Vector.mapM fromValue v
     fromValue _ = Nothing
 
-vectorItemType :: IsValue a => Vector a -> Type
-vectorItemType v = typeOf (undefined `asTypeOf` Data.Vector.head v)
-
 instance IsValue a => IsVariant (Vector a) where
     toVariant = Variant . toValue
     fromVariant (Variant val) = fromValue val
 
 instance IsValue a => IsValue [a] where
-    typeOf v = TypeArray (typeOf (undefined `asTypeOf` head v))
+    typeOf' _ = TypeArray (typeOf' (Proxy :: Proxy a))
     toValue = toValue . Data.Vector.fromList
     fromValue = fmap Data.Vector.toList . fromValue
 
@@ -556,35 +539,37 @@ instance IsValue a => IsVariant [a] where
     toVariant = toVariant . Data.Vector.fromList
     fromVariant = fmap Data.Vector.toList . fromVariant
 
-instance IsValue ByteString where
-    typeOf _ = TypeArray TypeWord8
+instance IsValue BS.ByteString where
+    typeOf' _ = TypeArray TypeWord8
     toValue = ValueBytes
     fromValue (ValueBytes bs) = Just bs
     fromValue (ValueVector TypeWord8 v) = Just (vectorToBytes v)
     fromValue _ = Nothing
 
-instance IsVariant ByteString where
+instance IsVariant BS.ByteString where
     toVariant = Variant . toValue
     fromVariant (Variant val) = fromValue val
 
-instance IsValue Data.ByteString.Lazy.ByteString where
-    typeOf _ = TypeArray TypeWord8
+instance IsValue BL.ByteString where
+    typeOf' _ = TypeArray TypeWord8
     toValue = toValue
-            . Data.ByteString.concat
-            . Data.ByteString.Lazy.toChunks
-    fromValue = fmap (\bs -> Data.ByteString.Lazy.fromChunks [bs])
+            . BS.concat
+            . BL.toChunks
+    fromValue = fmap (\bs -> BL.fromChunks [bs])
               . fromValue
 
-instance IsVariant Data.ByteString.Lazy.ByteString where
+instance IsVariant BL.ByteString where
     toVariant = Variant . toValue
     fromVariant (Variant val) = fromValue val
 
 instance (Ord k, IsAtom k, IsValue v) => IsValue (Map k v) where
-    typeOf m = TypeDictionary kt vt where
-        (kt, vt) = mapItemType m
+    typeOf' _ = TypeDictionary
+        (typeOf' (Proxy :: Proxy k))
+        (typeOf' (Proxy :: Proxy v))
 
     toValue m = ValueMap kt vt (bimap box m) where
-        (kt, vt) = mapItemType m
+        kt = typeOf' (Proxy :: Proxy k)
+        vt = typeOf' (Proxy :: Proxy v)
         box k v = (toAtom k, toValue v)
 
     fromValue (ValueMap _ _ m) = bimapM unbox m where
@@ -600,18 +585,12 @@ bimap f = Data.Map.fromList . map (\(k, v) -> f k v) . Data.Map.toList
 bimapM :: (Monad m, Ord k') => (k -> v -> m (k', v')) -> Map k v -> m (Map k' v')
 bimapM f = liftM Data.Map.fromList . mapM (\(k, v) -> f k v) . Data.Map.toList
 
-mapItemType :: (IsValue k, IsValue v) => Map k v -> (Type, Type)
-mapItemType m = (typeOf k, typeOf v) where
-    mapItem :: Map k v -> (k, v)
-    mapItem _ = (undefined, undefined)
-    (k, v) = mapItem m
-
 instance (Ord k, IsAtom k, IsValue v) => IsVariant (Map k v) where
     toVariant = Variant . toValue
     fromVariant (Variant val) = fromValue val
 
 instance IsValue () where
-  typeOf _ = TypeStructure []
+  typeOf' _ = TypeStructure []
   toValue _ = ValueStructure []
   fromValue (ValueStructure []) = return ()
   fromValue _ = Nothing
@@ -622,7 +601,10 @@ instance IsVariant () where
   fromVariant _ = Nothing
 
 instance (IsValue a1, IsValue a2) => IsValue (a1, a2) where
-    typeOf ~(a1, a2) = TypeStructure [typeOf a1, typeOf a2]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        ]
     toValue (a1, a2) = ValueStructure [toValue a1, toValue a2]
     fromValue (ValueStructure [a1, a2]) = do
         a1' <- fromValue a1
@@ -663,7 +645,7 @@ fromElements elems = objectPath_ $ '/':intercalate "/" elems
 formatObjectPath :: ObjectPath -> String
 formatObjectPath (ObjectPath s) = s
 
-parseObjectPath :: String -> Maybe ObjectPath
+parseObjectPath :: MonadThrow m => String -> m ObjectPath
 parseObjectPath s = do
     maybeParseString parserObjectPath s
     return (ObjectPath s)
@@ -706,9 +688,9 @@ newtype InterfaceName = InterfaceName String
 formatInterfaceName :: InterfaceName -> String
 formatInterfaceName (InterfaceName s) = s
 
-parseInterfaceName :: String -> Maybe InterfaceName
+parseInterfaceName :: MonadThrow m => String -> m InterfaceName
 parseInterfaceName s = do
-    when (length s > 255) Nothing
+    when (length s > 255) $ throwM $ userError "name too long"
     maybeParseString parserInterfaceName s
     return (InterfaceName s)
 
@@ -746,9 +728,9 @@ newtype MemberName = MemberName String
 formatMemberName :: MemberName -> String
 formatMemberName (MemberName s) = s
 
-parseMemberName :: String -> Maybe MemberName
+parseMemberName :: MonadThrow m => String -> m MemberName
 parseMemberName s = do
-    when (length s > 255) Nothing
+    when (length s > 255) $ throwM $ userError "name too long"
     maybeParseString parserMemberName s
     return (MemberName s)
 
@@ -783,9 +765,9 @@ newtype ErrorName = ErrorName String
 formatErrorName :: ErrorName -> String
 formatErrorName (ErrorName s) = s
 
-parseErrorName :: String -> Maybe ErrorName
+parseErrorName :: MonadThrow m => String -> m ErrorName
 parseErrorName s = do
-    when (length s > 255) Nothing
+    when (length s > 255) $ throwM $ userError "name too long"
     maybeParseString parserInterfaceName s
     return (ErrorName s)
 
@@ -813,9 +795,9 @@ newtype BusName = BusName String
 formatBusName :: BusName -> String
 formatBusName (BusName s) = s
 
-parseBusName :: String -> Maybe BusName
+parseBusName :: MonadThrow m => String -> m BusName
 parseBusName s = do
-    when (length s > 255) Nothing
+    when (length s > 255) $ throwM $ userError "name too long"
     maybeParseString parserBusName s
     return (BusName s)
 
@@ -883,7 +865,7 @@ structureItems (Structure xs) = map Variant xs
 -- 'IsValue'.
 data Array
     = Array Type (Vector Value)
-    | ArrayBytes ByteString
+    | ArrayBytes BS.ByteString
 
 instance Show Array where
     show (Array t xs) = showValue True (ValueVector t xs)
@@ -904,7 +886,7 @@ instance IsVariant Array where
 
 arrayItems :: Array -> [Variant]
 arrayItems (Array _ xs) = map Variant (Data.Vector.toList xs)
-arrayItems (ArrayBytes bs) = map toVariant (Data.ByteString.unpack bs)
+arrayItems (ArrayBytes bs) = map toVariant (BS.unpack bs)
 
 -- | A D-Bus Dictionary is a container type similar to Haskell maps, storing
 -- zero or more associations between keys and values.
@@ -930,7 +912,11 @@ dictionaryItems (Dictionary _ _ xs) = do
     return (Variant (ValueAtom k), Variant v)
 
 instance (IsValue a1, IsValue a2, IsValue a3) => IsValue (a1, a2, a3) where
-    typeOf ~(a1, a2, a3) = TypeStructure [typeOf a1, typeOf a2, typeOf a3]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        ]
     toValue (a1, a2, a3) = ValueStructure [toValue a1, toValue a2, toValue a3]
     fromValue (ValueStructure [a1, a2, a3]) = do
         a1' <- fromValue a1
@@ -940,7 +926,12 @@ instance (IsValue a1, IsValue a2, IsValue a3) => IsValue (a1, a2, a3) where
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4) => IsValue (a1, a2, a3, a4) where
-    typeOf ~(a1, a2, a3, a4) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        ]
     toValue (a1, a2, a3, a4) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4]
     fromValue (ValueStructure [a1, a2, a3, a4]) = do
         a1' <- fromValue a1
@@ -951,7 +942,13 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4) => IsValue (a1, a2, a3
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5) => IsValue (a1, a2, a3, a4, a5) where
-    typeOf ~(a1, a2, a3, a4, a5) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        ]
     toValue (a1, a2, a3, a4, a5) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5]
     fromValue (ValueStructure [a1, a2, a3, a4, a5]) = do
         a1' <- fromValue a1
@@ -963,7 +960,14 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5) => IsValue
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6) => IsValue (a1, a2, a3, a4, a5, a6) where
-    typeOf ~(a1, a2, a3, a4, a5, a6) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        ]
     toValue (a1, a2, a3, a4, a5, a6) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6]) = do
         a1' <- fromValue a1
@@ -976,7 +980,15 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7) => IsValue (a1, a2, a3, a4, a5, a6, a7) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7]) = do
         a1' <- fromValue a1
@@ -990,7 +1002,16 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8]) = do
         a1' <- fromValue a1
@@ -1005,7 +1026,17 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9]) = do
         a1' <- fromValue a1
@@ -1021,7 +1052,18 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9, IsValue a10) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9, typeOf a10]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        , typeOf' (Proxy :: Proxy a10)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9, toValue a10]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9, a10]) = do
         a1' <- fromValue a1
@@ -1038,7 +1080,19 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9, IsValue a10, IsValue a11) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9, typeOf a10, typeOf a11]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        , typeOf' (Proxy :: Proxy a10)
+        , typeOf' (Proxy :: Proxy a11)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9, toValue a10, toValue a11]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11]) = do
         a1' <- fromValue a1
@@ -1056,7 +1110,20 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9, IsValue a10, IsValue a11, IsValue a12) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9, typeOf a10, typeOf a11, typeOf a12]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        , typeOf' (Proxy :: Proxy a10)
+        , typeOf' (Proxy :: Proxy a11)
+        , typeOf' (Proxy :: Proxy a12)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9, toValue a10, toValue a11, toValue a12]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12]) = do
         a1' <- fromValue a1
@@ -1075,7 +1142,21 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9, IsValue a10, IsValue a11, IsValue a12, IsValue a13) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9, typeOf a10, typeOf a11, typeOf a12, typeOf a13]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        , typeOf' (Proxy :: Proxy a10)
+        , typeOf' (Proxy :: Proxy a11)
+        , typeOf' (Proxy :: Proxy a12)
+        , typeOf' (Proxy :: Proxy a13)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9, toValue a10, toValue a11, toValue a12, toValue a13]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13]) = do
         a1' <- fromValue a1
@@ -1095,7 +1176,22 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9, IsValue a10, IsValue a11, IsValue a12, IsValue a13, IsValue a14) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9, typeOf a10, typeOf a11, typeOf a12, typeOf a13, typeOf a14]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        , typeOf' (Proxy :: Proxy a10)
+        , typeOf' (Proxy :: Proxy a11)
+        , typeOf' (Proxy :: Proxy a12)
+        , typeOf' (Proxy :: Proxy a13)
+        , typeOf' (Proxy :: Proxy a14)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9, toValue a10, toValue a11, toValue a12, toValue a13, toValue a14]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14]) = do
         a1' <- fromValue a1
@@ -1116,7 +1212,23 @@ instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6
     fromValue _ = Nothing
 
 instance (IsValue a1, IsValue a2, IsValue a3, IsValue a4, IsValue a5, IsValue a6, IsValue a7, IsValue a8, IsValue a9, IsValue a10, IsValue a11, IsValue a12, IsValue a13, IsValue a14, IsValue a15) => IsValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15) where
-    typeOf ~(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15) = TypeStructure [typeOf a1, typeOf a2, typeOf a3, typeOf a4, typeOf a5, typeOf a6, typeOf a7, typeOf a8, typeOf a9, typeOf a10, typeOf a11, typeOf a12, typeOf a13, typeOf a14, typeOf a15]
+    typeOf' _ = TypeStructure
+        [ typeOf' (Proxy :: Proxy a1)
+        , typeOf' (Proxy :: Proxy a2)
+        , typeOf' (Proxy :: Proxy a3)
+        , typeOf' (Proxy :: Proxy a4)
+        , typeOf' (Proxy :: Proxy a5)
+        , typeOf' (Proxy :: Proxy a6)
+        , typeOf' (Proxy :: Proxy a7)
+        , typeOf' (Proxy :: Proxy a8)
+        , typeOf' (Proxy :: Proxy a9)
+        , typeOf' (Proxy :: Proxy a10)
+        , typeOf' (Proxy :: Proxy a11)
+        , typeOf' (Proxy :: Proxy a12)
+        , typeOf' (Proxy :: Proxy a13)
+        , typeOf' (Proxy :: Proxy a14)
+        , typeOf' (Proxy :: Proxy a15)
+        ]
     toValue (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15) = ValueStructure [toValue a1, toValue a2, toValue a3, toValue a4, toValue a5, toValue a6, toValue a7, toValue a8, toValue a9, toValue a10, toValue a11, toValue a12, toValue a13, toValue a14, toValue a15]
     fromValue (ValueStructure [a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15]) = do
         a1' <- fromValue a1
@@ -1365,9 +1477,9 @@ forceParse label parse str = case parse str of
     Just x -> x
     Nothing -> error ("Invalid " ++ label ++ ": " ++ show str)
 
-maybeParseString :: Parsec.Parser a -> String -> Maybe a
+maybeParseString :: MonadThrow m => Parsec.Parser a -> String -> m a
 maybeParseString parser s = case Parsec.parse parser "" s of
-    Left _ -> Nothing
-    Right a -> Just a
+    Left err -> throwM $ userError $ show err
+    Right a -> pure a
 
 THL.deriveLiftMany [''BusName, ''ObjectPath, ''InterfaceName, ''MemberName]
