@@ -1,3 +1,4 @@
+{-# LANGUAGE OverloadedStrings #-}
 -- Copyright (C) 2010-2012 John Millikin <john@john-millikin.com>
 --
 -- Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,9 +21,12 @@ import Test.QuickCheck
 import Test.Tasty
 import Test.Tasty.HUnit
 import Test.Tasty.QuickCheck
+import qualified Data.Text as T
 
 import DBus
-import qualified DBus.Introspection as I
+import qualified DBus.Introspection.Parse as I
+import qualified DBus.Introspection.Render as I
+import qualified DBus.Introspection.Types as I
 
 import DBusTests.InterfaceName ()
 import DBusTests.MemberName ()
@@ -42,7 +46,7 @@ test_XmlPassthrough :: TestTree
 test_XmlPassthrough = testProperty "xml-passthrough" $ \obj -> let
     path = I.objectPath obj
     Just xml = I.formatXML obj
-    in I.parseXML path xml == Just obj
+    in I.parseXML path (T.pack xml) == Just obj
 
 buildEmptyObject :: String -> I.Object
 buildEmptyObject name = I.Object (objectPath_ name) [] []
